<?php

if (!defined('PHORUM')) return;

// ----------------------------------------------------------------------
// Hook functions.
// ----------------------------------------------------------------------

// Rewrite displaynames to usernames for PM recipients and search by author.
function mod_real_name_common()
{
    if (phorum_page == 'pm') {
        if (isset($_POST['to_name']) && trim($_POST['to_name']) != '') {
            $_POST['to_name'] = mod_real_name_rewrite(
                'displayname2username',
                trim($_POST['to_name'])
            );
        }
    }

    if (phorum_page == 'search') {
        if (isset($_GET['search']) && trim($_GET['search']) != '' &&
            isset($_GET['match_type']) && $_GET['match_type'] == 'AUTHOR') {
            $_GET['search'] = mod_real_name_rewrite(
                'displayname2username',
                trim($_GET['search'])
            );
        }
    }
}

// Apply the rewrite rules to the Phorum data.
// Write query string for author search back to the real name.
function mod_real_name_after_header() 
{
    global $PHORUM;

    // The list of input files containing rewrite rules. This list can
    // be modified by modules. See also the README for this module.
    $files = array(
        "./mods/real_name/rewrite_rules/core.src",
        "./mods/real_name/rewrite_rules/modules.src"
    );
    $files = phorum_hook("real_name_add_rules", $files);

    // To keep cache items from multiple Phorum trees separated.
    $cache_base = $PHORUM["cache"] . "/rewrite_rules." . md5(getcwd());

    // Check if we need to recompile the rules file(s).
    $recompile = false;
    foreach ($files as $file) {
        $id = md5($file);
        $stamp = "{$cache_base}.{$id}.timestamp";
        if (!file_exists($stamp) || filemtime($stamp) < filemtime($file)) {
            $recompile = true;
            break;
        }
    }

    // Compile the rules file into PHP code.
    if ($recompile)
    {
        require_once("rewrite_rules_compiler.php");
        $compiler = new rewrite_rules_compiler();
        $code = array();

        foreach ($files as $file)
        {
            // Compile the code from the current rules source file.
            $code = $compiler->compile($file, $code); 

            // Create a timestamp file for the current rules source file.
            $id = md5($file);
            $stamp = "{$cache_base}.{$id}.timestamp";
            $touch = fopen($stamp, "w");
            if (! $touch)
                die("Can't write file " . htmlspecialchars($stamp));
            if (! fclose($touch)) 
                die("Close error on " . htmlspecialchars($stamp) . ". " .
                    "Is the disk full?");
        }

        // Write the compiled code per phorum_page to disk.
        foreach ($code as $page => $chunk)
        {
            // If there is no compiled code for the current page,
            // then continue with the next.
            if ($chunk == '') continue;

            // The empty page name is for global rules.
            // We prepend those to each code cache file and don't write
            // separate cache file for them.
            if ($page == '') continue;

            // Write the code cache file for the current page.
            $codecache = "{$cache_base}.{$page}.php";
            $fp = fopen($codecache, "w");
            if (! $fp)
                die("Can't write file " . htmlspecialchars($codecache));
            fputs($fp, 
                "<?php\n" .
                ($code[''] != '' ? "// Global rules\n\n{$code['']}\n\n" : "") .
                "// Rules for phorum_page $page\n\n{$chunk}\n" .
                "?>");
            if (! fclose($fp)) 
                die("Close error on " . htmlspecialchars($codecache) . ". " .
                    "Is the disk full?");
        }
    }

    // Run the compiled rewrite rules for the current page.
    $codecache = $cache_base . "." . phorum_page . ".php";
    if (file_exists($codecache)) include($codecache);

    // Support for Phorum's search page and the search-user module.
    // Translate a username that has been searched for back to the
    // real name. Also handle translation of the "search all posts
    // by user" text.
    if (phorum_page == 'search' || phorum_page == 'search-user')
    {
        if (isset($PHORUM['args']['search']) &&
            $PHORUM['args']['search'] &&
            isset($PHORUM['DATA']['SEARCH']['match_type']) &&
            $PHORUM['DATA']['SEARCH']['match_type'] == 'AUTHOR') {

            // Find the user id for the username in the search.
            $username = $PHORUM['args']['search'];
            $user_id = phorum_db_user_check_field('username', $username);
            if (isset($user_id) && $user_id > 0) {
                $displayname = mod_real_name_rewrite(
                    'displayname',
                    $user_id
                );
            } else {
                $displayname = htmlspecialchars($username);
            }

            $PHORUM['DATA']['SEARCH']['safe_search'] = $displayname;
        }

        if (isset($PHORUM['args']['search']) &&
            isset($PHORUM['DATA']['SEARCH']['match_type']) &&
            $PHORUM['DATA']['SEARCH']['match_type'] == 'USER_ID' &&
            isset($PHORUM['DATA']['DESCRIPTION'])) {
            $display = mod_real_name_rewrite(
                'displayname',
                (int)$PHORUM['args']['search']
            );
            $PHORUM["DATA"]["DESCRIPTION"] = str_replace("%user%", htmlspecialchars($display), $PHORUM["DATA"]["LANG"]["SearchAllPostsHelp"]);
        }
    }
}

// Change the username in the edit notification after editing a message.
function mod_real_name_pre_edit($data)
{
    $PHORUM = $GLOBALS["PHORUM"];

    if (isset($PHORUM['user']['user_id']) && $PHORUM['user']['user_id']) {
        $data['meta']['edit_username'] = mod_real_name_rewrite(
            'displayname',
            $PHORUM["user"]["user_id"]
        );
    }

    return $data;
}

// Generate the information for the user drop down list.
function mod_real_name_user_list($userlist)
{
    if (is_array($userlist) && count($userlist) > 0) {
        foreach ($userlist as $user_id => $user) {
            $displayname = mod_real_name_rewrite('displayname', $user_id);
            $userlist[$user_id]["displayname"] = $displayname;
        }
    }

    $userlist = mod_real_name_array_sort($userlist, 'displayname');

    return $userlist;
}

function mod_real_name_quote($array)
{
    $PHORUM = $GLOBALS["PHORUM"];

    // No quote function configured.
    if (! isset($PHORUM["mod_real_name"]["quote_type"])) return $array;

    $username = $array[0];
    $message  = $array[1];

    // Handle BBcode style quoting.
    if ($PHORUM["mod_real_name"]["quote_type"] == "bbcode") {
        $displayname = mod_real_name_rewrite("username2displayname", $username);

        return "[quote {$displayname}]{$message}[/quote]";
    }

    // Handle default Phorum style quoting.
    elseif ($PHORUM["mod_real_name"]["quote_type"] == "built-in") {
        $displayname = mod_real_name_rewrite("username2displayname", $username);
        $quoted = phorum_strip_body($message);
        $quoted = str_replace("\n", "\n> ", $quoted);
        $quoted = wordwrap(trim($quoted), 50, "\n> ", true);
        $quoted = "{$displayname} {$PHORUM["DATA"]["LANG"]["Wrote"]}:\n" .
                  str_repeat("-", 55) . "\n> $quoted\n\n\n";
        return $quoted;
    }

    return $array;
}

// Handy little function for sorting.
// Thanks to acecream in the php manual.
function mod_real_name_array_sort($array, $key) {
    if (is_array($array) && count($array) > 0 && isset($key) && $key) {
        foreach ($array as $i => $k) {
            $sort_values[$i] = strtolower($array[$i][$key]);
        }

        asort($sort_values);

        while (list ($arr_key, $arr_val) = each ($sort_values)) {
             $sorted_arr[$arr_key] = $array[$arr_key];
        }
    } else {
        $sorted_arr = $array;
    }

    return $sorted_arr;
}


// ----------------------------------------------------------------------
// Handler for rewrite commands (both used from the compiled rewrite
// rules and hook functions).
// ----------------------------------------------------------------------

function mod_real_name_rewrite($command, $arg)
{
    // username2displayname : translates a username into a displayname
    if ($command == 'username2displayname') {
        $username = $arg;

        $user_id = phorum_db_user_check_field('username', $username);
        if (isset($user_id) && $user_id > 0) {
	    $command = 'displayname';
	    $arg     = $user_id;
	} else {
	    return $username;
	}
    }
 
    // profile_link : translates a user_id into a full clickable profile link
    // profile_url  : translates a user_id into a profile url
    // displayname  : translates a user_id into a displayname
    if ($command == 'profile_link' || 
        $command == 'displayname' || 
        $command == "profile_url") {

        $user_id     = $arg;
        $displayname = NULL;
        $link_url    = NULL;
        $link_html   = NULL;

        // If the $user_id is 0, this means we're handling an anonymous
        // posting user. In that case do not do any rewriting.
        if ($user_id == 0) return NULL;

        $user = phorum_user_get($user_id, false);

        // Determine the displayname to show.
        if (!$user) {
            // We have a user id in the posting, but the user does no longer
            // exist within forum. Use "anonmous user" for the display name.
            $displayname = htmlspecialchars($GLOBALS["PHORUM"]["DATA"]["LANG"]["AnonymousUser"]);
        } else {
            if (isset($user["real_name"]) && trim($user["real_name"]) != '') {

	        // Determine what the escaping setting for the real_name
		// field is. If the real name is not escaped by the 
		// config of the custom field, then we'll do it for
		// security reasons ourselves. If some admin really needs
		// to allow working HTML in real names, he's on its own :-)
		if (! isset($GLOBALS["PHORUM"]["real_name_is_escaped"])) {
		    foreach ($GLOBALS["PHORUM"]["PROFILE_FIELDS"] as $f) {
		        if ($f["name"] == "real_name") {
			   $GLOBALS["PHORUM"]["real_name_is_escaped"] = $f["html_disabled"];
			   break;
			}
		    }
		}
                $displayname = $GLOBALS["PHORUM"]["real_name_is_escaped"]
		             ? $user["real_name"]
			     : htmlspecialchars($user["real_name"]);
            } else {
                $displayname = htmlspecialchars($user["username"]);
            }
        }

        // Determine the profile page URL.
        if ($user) {
            $link_url = phorum_get_url(PHORUM_PROFILE_URL, $user_id);
        }

        // Allow other modules to provide user information.
	// The module must return the $displayname HTML escaped!
        if (isset($GLOBALS["PHORUM"]["hooks"]["real_name_userinfo"])) {
            list($user_id, $displayname, $link_url, $link_html) = phorum_hook(
                'real_name_userinfo',
                array($user_id, $displayname, $link_url, $link_html)
            );
        }

        switch ($command)
        {
            case 'profile_link':
              // Only build the html link if no module provided one.
              if ($link_html === NULL) {
                  $link_html = $displayname;
                  if ($link_url !== NULL) {
                      $link_html = '<a href="'.htmlspecialchars($link_url).'">'.
                                   $link_html . '</a>';
                  }
              }
              return $link_html;

            case 'profile_url':
              return htmlspecialchars($link_url);

            case 'displayname':
              return $displayname;
        }
    }

    // displayname2username : translates a displayname into a username
    elseif ($command == 'displayname2username')
    {
        $displayname = $arg;
        $username = NULL;

        // If the display name matches a username, that will get priority.
        $user_id = phorum_db_user_check_field('username', $displayname);
        if (isset($user_id) && $user_id > 0) $username = $displayname;

        // Try to find a username. If we won't find a username, we will go
        // with the current displayname.
        if ($username === NULL)
        {
            $username = $displayname;
            $user_id = NULL;

            // Try to find a unique username using an exact match 
            // on the real_name field.
            $ids = phorum_user_check_custom_field('real_name', $displayname);
            if (is_array($ids) && count($ids) == 1) {
                list($user_id) = each($ids);
            } else {
                // No user found. Try a like-clause.
                if (count($ids) == 0) {
                    $ids = phorum_user_check_custom_field('real_name', "%{$displayname}%", 1);
                    if (is_array($ids) && count($ids) == 1) {
                        list($user_id) = each($ids);
                    }
                }
            }
            if (isset($user_id) && $user_id > 0) {
                $user = phorum_user_get($user_id, false);
                $username = $user['username'];
            }
        }

        // Allow other modules to provide the translation.
        if (isset($GLOBALS["PHORUM"]["hooks"]["real_name_displayname2username"])) {
            list($displayname, $username) = phorum_hook(
                'real_name_displayname2username',
                array($displayname, $username)
            );
        }

        return htmlspecialchars($username);
    }

    else 
    {
        die("real_name mod: Illegal rewrite command issued: " .
            htmlspecialchars($command));
    }
}

?>
